@@ -1,7 +1,5 @@
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.64 $
-# Date: $Date: 2004/07/24 14:13:38 $
+# $Id: frontend.py 7339 2012-02-03 12:23:27Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -18,11 +16,13 @@ Exports the following classes:
 Also exports the following functions:
 
 * Option callbacks: `store_multiple`, `read_config_file`.
-* Setting validators (see `OptionParser.validators`: `validate_encoding`,
-  `validate_encoding_error_handler`, `validate_encoding_and_error_handler`,
-  `validate_boolean`, `validate_threshold`,
-  `validate_colon_separated_string_list`.
+* Setting validators: `validate_encoding`,
+  `validate_encoding_error_handler`,
+  `validate_encoding_and_error_handler`, `validate_boolean`,
+  `validate_threshold`, `validate_colon_separated_string_list`,
+  `validate_dependency_file`.
 * `make_paths_absolute`.
+* SettingSpec manipulation: `filter_settings_spec`.
 """
 
 __docformat__ = 'reStructuredText'
@@ -30,14 +30,15 @@ __docformat__ = 'reStructuredText'
 import os
 import os.path
 import sys
-import types
-import copy
 import warnings
 import ConfigParser as CP
 import codecs
-import docutils
 import optparse
-from optparse import Values, SUPPRESS_HELP
+from optparse import SUPPRESS_HELP
+import docutils
+import docutils.utils
+import docutils.nodes
+from docutils.error_reporting import locale_encoding, ErrorOutput, ErrorString
 
 
 def store_multiple(option, opt, value, parser, *args, **kwargs):
@@ -76,12 +77,6 @@ def validate_encoding_error_handler(setting, value, option_parser,
                                     config_parser=None, config_section=None):
     try:
         codecs.lookup_error(value)
-    except AttributeError:              # prior to Python 2.3
-        if value not in ('strict', 'ignore', 'replace', 'xmlcharrefreplace'):
-            raise (LookupError(
-                'unknown encoding error handler: "%s" (choices: '
-                '"strict", "ignore", "replace", or "xmlcharrefreplace")' % value),
-                   None, sys.exc_info()[2])
     except LookupError:
         raise (LookupError(
             'unknown encoding error handler: "%s" (choices: '
@@ -115,7 +110,7 @@ def validate_encoding_and_error_handler(
 
 def validate_boolean(setting, value, option_parser,
                      config_parser=None, config_section=None):
-    if isinstance(value, types.StringType):
+    if isinstance(value, unicode):
         try:
             return option_parser.booleans[value.strip().lower()]
         except KeyError:
@@ -123,6 +118,13 @@ def validate_boolean(setting, value, option_parser,
                    None, sys.exc_info()[2])
     return value
 
+def validate_nonnegative_int(setting, value, option_parser,
+                             config_parser=None, config_section=None):
+    value = int(value)
+    if value < 0:
+        raise ValueError('negative value; must be positive or zero')
+    return value
+
 def validate_threshold(setting, value, option_parser,
                        config_parser=None, config_section=None):
     try:
@@ -136,7 +138,7 @@ def validate_threshold(setting, value, option_parser,
 
 def validate_colon_separated_string_list(
     setting, value, option_parser, config_parser=None, config_section=None):
-    if isinstance(value, types.StringType):
+    if isinstance(value, unicode):
         value = value.split(':')
     else:
         last = value.pop()
@@ -152,6 +154,28 @@ def validate_url_trailing_slash(
     else:
         return value + '/'
 
+def validate_dependency_file(setting, value, option_parser,
+                             config_parser=None, config_section=None):
+    try:
+        return docutils.utils.DependencyList(value)
+    except IOError:
+        return docutils.utils.DependencyList(None)
+
+def validate_strip_class(setting, value, option_parser,
+                         config_parser=None, config_section=None):
+    # convert to list:
+    if isinstance(value, unicode):
+        value = [value]
+    class_values = filter(None, [v.strip() for v in value.pop().split(',')])
+    # validate:
+    for class_value in class_values:
+        normalized = docutils.nodes.make_id(class_value)
+        if class_value != normalized:
+            raise ValueError('invalid class value %r (perhaps %r?)'
+                             % (class_value, normalized))
+    value.extend(class_values)
+    return value
+
 def make_paths_absolute(pathdict, keys, base_path=None):
     """
     Interpret filesystem path settings relative to the `base_path` given.
@@ -160,11 +184,12 @@ def make_paths_absolute(pathdict, keys, base_path=None):
     `OptionParser.relative_path_settings`.
     """
     if base_path is None:
-        base_path = os.getcwd()
+        base_path = os.getcwdu() # type(base_path) == unicode
+        # to allow combining non-ASCII cwd with unicode values in `pathdict`
     for key in keys:
-        if pathdict.has_key(key):
+        if key in pathdict:
             value = pathdict[key]
-            if isinstance(value, types.ListType):
+            if isinstance(value, list):
                 value = [make_one_path_absolute(base_path, path)
                          for path in value]
             elif value:
@@ -174,6 +199,35 @@ def make_paths_absolute(pathdict, keys, base_path=None):
 def make_one_path_absolute(base_path, path):
     return os.path.abspath(os.path.join(base_path, path))
 
+def filter_settings_spec(settings_spec, *exclude, **replace):
+    """Return a copy of `settings_spec` excluding/replacing some settings.
+
+    `settings_spec` is a tuple of configuration settings with a structure
+    described for docutils.SettingsSpec.settings_spec.
+
+    Optional positional arguments are names of to-be-excluded settings.
+    Keyword arguments are option specification replacements.
+    (See the html4strict writer for an example.)
+    """
+    settings = list(settings_spec)
+    # every third item is a sequence of option tuples
+    for i in range(2, len(settings), 3):
+        newopts = []
+        for opt_spec in settings[i]:
+            # opt_spec is ("<help>", [<option strings>], {<keyword args>})
+            opt_name = [opt_string[2:].replace('-', '_')
+                        for opt_string in opt_spec[1]
+                            if opt_string.startswith('--')
+                       ][0]
+            if opt_name in exclude:
+                continue
+            if opt_name in replace.keys():
+                newopts.append(replace[opt_name])
+            else:
+                newopts.append(opt_spec)
+        settings[i] = tuple(newopts)
+    return tuple(settings)
+
 
 class Values(optparse.Values):
 
@@ -182,40 +236,55 @@ class Values(optparse.Values):
     Works in conjunction with the `OptionParser.lists` instance attribute.
     """
 
+    def __init__(self, *args, **kwargs):
+        optparse.Values.__init__(self, *args, **kwargs)
+        if (not hasattr(self, 'record_dependencies')
+            or self.record_dependencies is None):
+            # Set up dependency list, in case it is needed.
+            self.record_dependencies = docutils.utils.DependencyList()
+
     def update(self, other_dict, option_parser):
         if isinstance(other_dict, Values):
             other_dict = other_dict.__dict__
         other_dict = other_dict.copy()
         for setting in option_parser.lists.keys():
-            if (hasattr(self, setting) and other_dict.has_key(setting)):
+            if (hasattr(self, setting) and setting in other_dict):
                 value = getattr(self, setting)
                 if value:
                     value += other_dict[setting]
                     del other_dict[setting]
         self._update_loose(other_dict)
 
+    def copy(self):
+        """Return a shallow copy of `self`."""
+        return self.__class__(defaults=self.__dict__)
+
 
 class Option(optparse.Option):
 
+    ATTRS = optparse.Option.ATTRS + ['validator', 'overrides']
+
     def process(self, opt, value, values, parser):
         """
-        Call the validator function on applicable settings.
+        Call the validator function on applicable settings and
+        evaluate the 'overrides' option.
         Extends `optparse.Option.process`.
         """
         result = optparse.Option.process(self, opt, value, values, parser)
         setting = self.dest
         if setting:
-            value = getattr(values, setting)
-            validator = parser.validators.get(setting)
-            if validator:
+            if self.validator:
+                value = getattr(values, setting)
                 try:
-                    new_value = validator(setting, value, parser)
+                    new_value = self.validator(setting, value, parser)
                 except Exception, error:
                     raise (optparse.OptionValueError(
-                        'Error in option "%s":\n    %s: %s'
-                        % (opt, error.__class__.__name__, error)),
+                        'Error in option "%s":\n    %s'
+                        % (opt, ErrorString(error))),
                            None, sys.exc_info()[2])
                 setattr(values, setting, new_value)
+            if self.overrides:
+                setattr(values, self.overrides, None)
         return result
 
 
@@ -248,16 +317,17 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
               '0': 0, 'off': 0, 'no': 0, 'false': 0, '': 0}
     """Lookup table for boolean configuration file settings."""
 
-    if hasattr(codecs, 'backslashreplace_errors'):
-        default_error_encoding_error_handler = 'backslashreplace'
-    else:
-        default_error_encoding_error_handler = 'replace'
+    default_error_encoding = getattr(sys.stderr, 'encoding',
+                                     None) or locale_encoding or 'ascii'
+
+    default_error_encoding_error_handler = 'backslashreplace'
 
     settings_spec = (
         'General Docutils Options',
         None,
-        (('Include a "Generated by Docutils" credit and link at the end '
-          'of the document.',
+        (('Specify the document title as metadata.',
+          ['--title'], {}),
+         ('Include a "Generated by Docutils" credit and link.',
           ['--generator', '-g'], {'action': 'store_true',
                                   'validator': validate_boolean}),
          ('Do not include a generator credit.',
@@ -265,129 +335,138 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
          ('Include the date at the end of the document (UTC).',
           ['--date', '-d'], {'action': 'store_const', 'const': '%Y-%m-%d',
                              'dest': 'datestamp'}),
-         ('Include the time & date at the end of the document (UTC).',
+         ('Include the time & date (UTC).',
           ['--time', '-t'], {'action': 'store_const',
                              'const': '%Y-%m-%d %H:%M UTC',
                              'dest': 'datestamp'}),
          ('Do not include a datestamp of any kind.',
           ['--no-datestamp'], {'action': 'store_const', 'const': None,
                                'dest': 'datestamp'}),
-         ('Include a "View document source" link (relative to destination).',
+         ('Include a "View document source" link.',
           ['--source-link', '-s'], {'action': 'store_true',
                                     'validator': validate_boolean}),
-         ('Use the supplied <URL> verbatim for a "View document source" '
-          'link; implies --source-link.',
+         ('Use <URL> for a source link; implies --source-link.',
           ['--source-url'], {'metavar': '<URL>'}),
          ('Do not include a "View document source" link.',
           ['--no-source-link'],
           {'action': 'callback', 'callback': store_multiple,
            'callback_args': ('source_link', 'source_url')}),
-         ('Enable backlinks from section headers to table of contents '
-          'entries.  This is the default.',
+         ('Link from section headers to TOC entries.  (default)',
           ['--toc-entry-backlinks'],
           {'dest': 'toc_backlinks', 'action': 'store_const', 'const': 'entry',
            'default': 'entry'}),
-         ('Enable backlinks from section headers to the top of the table of '
-          'contents.',
+         ('Link from section headers to the top of the TOC.',
           ['--toc-top-backlinks'],
           {'dest': 'toc_backlinks', 'action': 'store_const', 'const': 'top'}),
          ('Disable backlinks to the table of contents.',
           ['--no-toc-backlinks'],
           {'dest': 'toc_backlinks', 'action': 'store_false'}),
-         ('Enable backlinks from footnotes and citations to their '
-          'references.  This is the default.',
+         ('Link from footnotes/citations to references. (default)',
           ['--footnote-backlinks'],
           {'action': 'store_true', 'default': 1,
            'validator': validate_boolean}),
          ('Disable backlinks from footnotes and citations.',
           ['--no-footnote-backlinks'],
           {'dest': 'footnote_backlinks', 'action': 'store_false'}),
-         ('Disable Docutils section numbering',
-          ['--no-section-numbering'],
-          {'action': 'store_false', 'dest': 'sectnum_xform',
+         ('Enable section numbering by Docutils.  (default)',
+          ['--section-numbering'],
+          {'action': 'store_true', 'dest': 'sectnum_xform',
            'default': 1, 'validator': validate_boolean}),
-         ('Set verbosity threshold; report system messages at or higher than '
-          '<level> (by name or number: "info" or "1", warning/2, error/3, '
-          'severe/4; also, "none" or "5").  Default is 2 (warning).',
+         ('Disable section numbering by Docutils.',
+          ['--no-section-numbering'],
+          {'action': 'store_false', 'dest': 'sectnum_xform'}),
+         ('Remove comment elements from the document tree.',
+          ['--strip-comments'],
+          {'action': 'store_true', 'validator': validate_boolean}),
+         ('Leave comment elements in the document tree. (default)',
+          ['--leave-comments'],
+          {'action': 'store_false', 'dest': 'strip_comments'}),
+         ('Remove all elements with classes="<class>" from the document tree. '
+          'Warning: potentially dangerous; use with caution. '
+          '(Multiple-use option.)',
+          ['--strip-elements-with-class'],
+          {'action': 'append', 'dest': 'strip_elements_with_classes',
+           'metavar': '<class>', 'validator': validate_strip_class}),
+         ('Remove all classes="<class>" attributes from elements in the '
+          'document tree. Warning: potentially dangerous; use with caution. '
+          '(Multiple-use option.)',
+          ['--strip-class'],
+          {'action': 'append', 'dest': 'strip_classes',
+           'metavar': '<class>', 'validator': validate_strip_class}),
+         ('Report system messages at or higher than <level>: "info" or "1", '
+          '"warning"/"2" (default), "error"/"3", "severe"/"4", "none"/"5"',
           ['--report', '-r'], {'choices': threshold_choices, 'default': 2,
                                'dest': 'report_level', 'metavar': '<level>',
                                'validator': validate_threshold}),
-         ('Report all system messages, info-level and higher.  (Same as '
-          '"--report=info".)',
-          ['--verbose', '-v'], {'action': 'store_const', 'const': 'info',
+         ('Report all system messages.  (Same as "--report=1".)',
+          ['--verbose', '-v'], {'action': 'store_const', 'const': 1,
                                 'dest': 'report_level'}),
-         ('Do not report any system messages.  (Same as "--report=none".)',
-          ['--quiet', '-q'], {'action': 'store_const', 'const': 'none',
+         ('Report no system messages.  (Same as "--report=5".)',
+          ['--quiet', '-q'], {'action': 'store_const', 'const': 5,
                               'dest': 'report_level'}),
-         ('Set the threshold (<level>) at or above which system messages are '
-          'converted to exceptions, halting execution immediately by '
-          'exiting (or propagating the exception if --traceback set).  '
-          'Levels as in --report.  Default is 4 (severe).',
+         ('Halt execution at system messages at or above <level>.  '
+          'Levels as in --report.  Default: 4 (severe).',
           ['--halt'], {'choices': threshold_choices, 'dest': 'halt_level',
                        'default': 4, 'metavar': '<level>',
                        'validator': validate_threshold}),
-         ('Same as "--halt=info": halt processing at the slightest problem.',
-          ['--strict'], {'action': 'store_const', 'const': 'info',
+         ('Halt at the slightest problem.  Same as "--halt=info".',
+          ['--strict'], {'action': 'store_const', 'const': 1,
                          'dest': 'halt_level'}),
-         ('Enable a non-zero exit status for normal exit if non-halting '
-          'system messages (at or above <level>) were generated.  Levels as '
-          'in --report.  Default is 5 (disabled).  Exit status is the maximum '
-          'system message level plus 10 (11 for INFO, etc.).',
+         ('Enable a non-zero exit status for non-halting system messages at '
+          'or above <level>.  Default: 5 (disabled).',
           ['--exit-status'], {'choices': threshold_choices,
                               'dest': 'exit_status_level',
                               'default': 5, 'metavar': '<level>',
                               'validator': validate_threshold}),
-         ('Report debug-level system messages and generate diagnostic output.',
+         ('Enable debug-level system messages and diagnostics.',
           ['--debug'], {'action': 'store_true', 'validator': validate_boolean}),
-         ('Do not report debug-level system messages or generate diagnostic '
-          'output.',
+         ('Disable debug output.  (default)',
           ['--no-debug'], {'action': 'store_false', 'dest': 'debug'}),
-         ('Send the output of system messages (warnings) to <file>.',
+         ('Send the output of system messages to <file>.',
           ['--warnings'], {'dest': 'warning_stream', 'metavar': '<file>'}),
-         ('Enable Python tracebacks when halt-level system messages and '
-          'other exceptions occur.  Useful for debugging, and essential for '
-          'issue reports.',
+         ('Enable Python tracebacks when Docutils is halted.',
           ['--traceback'], {'action': 'store_true', 'default': None,
                             'validator': validate_boolean}),
-         ('Disable Python tracebacks when errors occur; report just the error '
-          'instead.  This is the default.',
+         ('Disable Python tracebacks.  (default)',
           ['--no-traceback'], {'dest': 'traceback', 'action': 'store_false'}),
-         ('Specify the encoding of input text.  Default is locale-dependent.',
+         ('Specify the encoding and optionally the '
+          'error handler of input text.  Default: <locale-dependent>:strict.',
           ['--input-encoding', '-i'],
-          {'metavar': '<name>', 'validator': validate_encoding}),
-         ('Specify the text encoding for output.  Default is UTF-8.  '
-          'Optionally also specify the error handler for unencodable '
-          'characters, after a colon (":"); default is "strict".  (See '
-          '"--output-encoding-error-encoding".)',
+          {'metavar': '<name[:handler]>',
+           'validator': validate_encoding_and_error_handler}),
+         ('Specify the error handler for undecodable characters.  '
+          'Choices: "strict" (default), "ignore", and "replace".',
+          ['--input-encoding-error-handler'],
+          {'default': 'strict', 'validator': validate_encoding_error_handler}),
+         ('Specify the text encoding and optionally the error handler for '
+          'output.  Default: UTF-8:strict.',
           ['--output-encoding', '-o'],
           {'metavar': '<name[:handler]>', 'default': 'utf-8',
            'validator': validate_encoding_and_error_handler}),
-         ('Specify the error handler for unencodable characters in '
-          'the output.  Acceptable values include "strict", "ignore", '
-          '"replace", "xmlcharrefreplace", and '
-          '"backslashreplace" (in Python 2.3+).  Default is "strict".  '
-          'Usually specified as part of --output-encoding.',
+         ('Specify error handler for unencodable output characters; '
+          '"strict" (default), "ignore", "replace", '
+          '"xmlcharrefreplace", "backslashreplace".',
           ['--output-encoding-error-handler'],
           {'default': 'strict', 'validator': validate_encoding_error_handler}),
-         ('Specify the text encoding for error output.  Default is ASCII.  '
-          'Optionally also specify the error handler for unencodable '
-          'characters, after a colon (":"); default is "%s".  (See '
-          '"--output-encoding-error-encoding".'
-          % default_error_encoding_error_handler,
+         ('Specify text encoding and error handler for error output.  '
+          'Default: %s:%s.'
+          % (default_error_encoding, default_error_encoding_error_handler),
           ['--error-encoding', '-e'],
-          {'metavar': '<name[:handler]>', 'default': 'ascii',
+          {'metavar': '<name[:handler]>', 'default': default_error_encoding,
            'validator': validate_encoding_and_error_handler}),
          ('Specify the error handler for unencodable characters in '
-          'error output.  See --output-encoding-error-handler for acceptable '
-          'values.  Default is "%s".  Usually specified as part of '
-          '--error-encoding.' % default_error_encoding_error_handler,
+          'error output.  Default: %s.'
+          % default_error_encoding_error_handler,
           ['--error-encoding-error-handler'],
           {'default': default_error_encoding_error_handler,
            'validator': validate_encoding_error_handler}),
-         ('Specify the language of input text (ISO 639 2-letter identifier).'
-          '  Default is "en" (English).',
+         ('Specify the language (as BCP 47 language tag).  Default: en.',
           ['--language', '-l'], {'dest': 'language_code', 'default': 'en',
                                  'metavar': '<name>'}),
+         ('Write output file dependencies to <file>.',
+          ['--record-dependencies'],
+          {'metavar': '<file>', 'validator': validate_dependency_file,
+           'default': None}),           # default set in Values class
          ('Read configuration settings from <file>, if it exists.',
           ['--config'], {'metavar': '<file>', 'type': 'string',
                          'action': 'callback', 'callback': read_config_file}),
@@ -395,6 +474,9 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
           ['--version', '-V'], {'action': 'version'}),
          ('Show this help message and exit.',
           ['--help', '-h'], {'action': 'help'}),
+         # Typically not useful for non-programmatical use:
+         (SUPPRESS_HELP, ['--id-prefix'], {'default': ''}),
+         (SUPPRESS_HELP, ['--auto-id-prefix'], {'default': 'id'}),
          # Hidden options, for development use only:
          (SUPPRESS_HELP, ['--dump-settings'], {'action': 'store_true'}),
          (SUPPRESS_HELP, ['--dump-internals'], {'action': 'store_true'}),
@@ -402,21 +484,26 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
          (SUPPRESS_HELP, ['--dump-pseudo-xml'], {'action': 'store_true'}),
          (SUPPRESS_HELP, ['--expose-internal-attribute'],
           {'action': 'append', 'dest': 'expose_internals',
-           'validator': validate_colon_separated_string_list}),))
+           'validator': validate_colon_separated_string_list}),
+         (SUPPRESS_HELP, ['--strict-visitor'], {'action': 'store_true'}),
+         ))
     """Runtime settings and command-line options common to all Docutils front
     ends.  Setting specs specific to individual Docutils components are also
     used (see `populate_from_components()`)."""
 
     settings_defaults = {'_disable_config': None,
                          '_source': None,
-                         '_destination': None}
+                         '_destination': None,
+                         '_config_files': None}
     """Defaults for settings that don't have command-line option equivalents."""
 
     relative_path_settings = ('warning_stream',)
 
     config_section = 'general'
 
-    version_template = '%%prog (Docutils %s)' % docutils.__version__
+    version_template = ('%%prog (Docutils %s [%s], Python %s, on %s)'
+                        % (docutils.__version__, docutils.__version_details__,
+                           sys.version.split()[0], sys.platform))
     """Default version message."""
 
     def __init__(self, components=(), defaults=None, read_config_files=None,
@@ -426,18 +513,13 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         ``.settings_spec`` attribute.  `defaults` is a mapping of setting
         default overrides.
         """
-        self.validators = {}
-        """{setting: validation function} mapping, used by `validate_options`.
-        Validation functions take three or five parameters: setting name,
-        value, an `OptionParser` (``self``), and a `ConfigParser` and config
-        file section if activated from a config file.  They return a (possibly
-        modified) value, or raise an exception.  Populated from the "validator"
-        keyword argument dictionary entries of components' ``settings_spec``
-        attribute."""
 
         self.lists = {}
         """Set of list-type settings."""
 
+        self.config_files = []
+        """List of paths of applied configuration files."""
+
         optparse.OptionParser.__init__(
             self, option_class=Option, add_help_option=None,
             formatter=optparse.TitledHelpFormatter(width=78),
@@ -448,13 +530,13 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         self.relative_path_settings = list(self.relative_path_settings)
         self.components = (self,) + tuple(components)
         self.populate_from_components(self.components)
-        self.set_defaults(**(defaults or {}))
+        self.set_defaults_from_dict(defaults or {})
         if read_config_files and not self.defaults['_disable_config']:
             try:
                 config_settings = self.get_standard_config_settings()
             except ValueError, error:
                 self.error(error)
-            self.set_defaults(**config_settings.__dict__)
+            self.set_defaults_from_dict(config_settings.__dict__)
 
     def populate_from_components(self, components):
         """
@@ -477,16 +559,8 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
                 else:
                     group = self        # single options
                 for (help_text, option_strings, kwargs) in option_spec:
-                    kwargs = kwargs.copy() # to be modified, locally only
-                    if kwargs.has_key('validator'):
-                        validator = kwargs['validator']
-                        del kwargs['validator']
-                    else:
-                        validator = None
                     option = group.add_option(help=help_text, *option_strings,
                                               **kwargs)
-                    if validator:
-                        self.validators[option.dest] = validator
                     if kwargs.get('action') == 'append':
                         self.lists[option.dest] = 1
                 if component.settings_defaults:
@@ -501,7 +575,19 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
             config_files = os.environ['DOCUTILSCONFIG'].split(os.pathsep)
         except KeyError:
             config_files = self.standard_config_files
-        return [os.path.expanduser(f) for f in config_files if f.strip()]
+
+        # If 'HOME' is not set, expandvars() requires the 'pwd' module which is
+        # not available under certain environments, for example, within
+        # mod_python.  The publisher ends up in here, and we need to publish
+        # from within mod_python.  Therefore we need to avoid expanding when we
+        # are in those environments.
+        expand = os.path.expanduser
+        if 'HOME' not in os.environ:
+            try:
+                import pwd
+            except ImportError:
+                expand = lambda x: x
+        return [expand(f) for f in config_files if f.strip()]
 
     def get_standard_config_settings(self):
         settings = Values()
@@ -513,6 +599,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         """Returns a dictionary containing appropriate config file settings."""
         parser = ConfigParser()
         parser.read(config_file, self)
+        self.config_files.extend(parser._files)
         base_path = os.path.dirname(config_file)
         applied = {}
         settings = Values()
@@ -521,7 +608,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
                 continue
             for section in (tuple(component.config_section_dependencies or ())
                             + (component.config_section,)):
-                if applied.has_key(section):
+                if section in applied:
                     continue
                 applied[section] = 1
                 settings.update(parser.get_section(section), self)
@@ -532,8 +619,8 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
     def check_values(self, values, args):
         """Store positional arguments as runtime settings."""
         values._source, values._destination = self.check_args(args)
-        make_paths_absolute(values.__dict__, self.relative_path_settings,
-                            os.getcwd())
+        make_paths_absolute(values.__dict__, self.relative_path_settings)
+        values._config_files = self.config_files
         return values
 
     def check_args(self, args):
@@ -553,12 +640,33 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
                        'destination.  It will clobber the source file.')
         return source, destination
 
+    def set_defaults_from_dict(self, defaults):
+        self.defaults.update(defaults)
+
     def get_default_values(self):
         """Needed to get custom `Values` instances."""
-        return Values(self.defaults)
+        defaults = Values(self.defaults)
+        defaults._config_files = self.config_files
+        return defaults
+
+    def get_option_by_dest(self, dest):
+        """
+        Get an option by its dest.
+
+        If you're supplying a dest which is shared by several options,
+        it is undefined which option of those is returned.
+
+        A KeyError is raised if there is no option with the supplied
+        dest.
+        """
+        for group in self.option_groups + [self]:
+            for option in group.option_list:
+                if option.dest == dest:
+                    return option
+        raise KeyError('No option with dest == %r.' % dest)
 
 
-class ConfigParser(CP.ConfigParser):
+class ConfigParser(CP.RawConfigParser):
 
     old_settings = {
         'pep_stylesheet': ('pep_html writer', 'stylesheet'),
@@ -574,11 +682,40 @@ configuration files.  See <http://docutils.sf.net/docs/user/config.html>,
 section "Old-Format Configuration Files".
 """
 
+    not_utf8_error = """\
+Unable to read configuration file "%s": content not encoded as UTF-8.
+Skipping "%s" configuration file.
+"""
+
+    def __init__(self, *args, **kwargs):
+        CP.RawConfigParser.__init__(self, *args, **kwargs)
+
+        self._files = []
+        """List of paths of configuration files read."""
+
+        self._stderr = ErrorOutput()
+        """Wrapper around sys.stderr catching en-/decoding errors"""
+
     def read(self, filenames, option_parser):
-        if type(filenames) in (types.StringType, types.UnicodeType):
+        if type(filenames) in (str, unicode):
             filenames = [filenames]
         for filename in filenames:
-            CP.ConfigParser.read(self, filename)
+            try:
+                # Config files must be UTF-8-encoded:
+                fp = codecs.open(filename, 'r', 'utf-8')
+            except IOError:
+                continue
+            try:
+                if sys.version_info < (3,2):
+                    CP.RawConfigParser.readfp(self, fp, filename)
+                else:
+                    CP.RawConfigParser.read_file(self, fp, filename)
+            except UnicodeDecodeError:
+                self._stderr.write(self.not_utf8_error % (filename, filename))
+                fp.close()
+                continue
+            fp.close()
+            self._files.append(filename)
             if self.has_section('options'):
                 self.handle_old_config(filename)
             self.validate_settings(filename, option_parser)
@@ -590,7 +727,7 @@ section "Old-Format Configuration Files".
         if not self.has_section('general'):
             self.add_section('general')
         for key, value in options.items():
-            if self.old_settings.has_key(key):
+            if key in self.old_settings:
                 section, setting = self.old_settings[key]
                 if not self.has_section(section):
                     self.add_section(section)
@@ -602,23 +739,32 @@ section "Old-Format Configuration Files".
         self.remove_section('options')
 
     def validate_settings(self, filename, option_parser):
-        """Call the validator function on all applicable settings."""
+        """
+        Call the validator function and implement overrides on all applicable
+        settings.
+        """
         for section in self.sections():
             for setting in self.options(section):
-                validator = option_parser.validators.get(setting)
-                if validator:
-                    value = self.get(section, setting, raw=1)
+                try:
+                    option = option_parser.get_option_by_dest(setting)
+                except KeyError:
+                    continue
+                if option.validator:
+                    value = self.get(section, setting)
                     try:
-                        new_value = validator(
+                        new_value = option.validator(
                             setting, value, option_parser,
                             config_parser=self, config_section=section)
                     except Exception, error:
                         raise (ValueError(
                             'Error in config file "%s", section "[%s]":\n'
-                            '    %s: %s\n        %s = %s'
-                            % (filename, section, error.__class__.__name__,
-                               error, setting, value)), None, sys.exc_info()[2])
+                            '    %s\n'
+                            '        %s = %s'
+                            % (filename, section, ErrorString(error),
+                               setting, value)), None, sys.exc_info()[2])
                     self.set(section, setting, new_value)
+                if option.overrides:
+                    self.set(section, option.overrides, None)
 
     def optionxform(self, optionstr):
         """
@@ -634,7 +780,7 @@ section "Old-Format Configuration Files".
         section_dict = {}
         if self.has_section(section):
             for option in self.options(section):
-                section_dict[option] = self.get(section, option, raw=1)
+                section_dict[option] = self.get(section, option)
         return section_dict
 
 
