@@ -1,4 +1,4 @@
-# $Id: DocutilsTestSupport.py 5033 2007-03-21 19:51:22Z wiemann $
+# $Id: DocutilsTestSupport.py 7400 2012-05-01 08:34:41Z grubert $
 # Authors: David Goodger <goodger@python.org>;
 #          Garth Kidd <garth@deadlybloodyserious.com>
 # Copyright: This module has been placed in the public domain.
@@ -47,7 +47,6 @@ import re
 import inspect
 import traceback
 from pprint import pformat
-from types import UnicodeType, StringType
 
 testroot = os.path.abspath(os.path.dirname(__file__) or os.curdir)
 os.chdir(testroot)
@@ -56,7 +55,7 @@ sys.path.insert(0, testroot)
 sys.path.append(os.path.normpath(os.path.join(testroot, '..', 'extras')))
 
 try:
-    import docutils_difflib
+    import difflib
     import package_unittest
     import docutils
     import docutils.core
@@ -65,8 +64,8 @@ try:
     from docutils.parsers import rst
     from docutils.parsers.rst import states, tableparser, roles, languages
     from docutils.readers import standalone, pep
-    from docutils.readers.python import moduleparser
     from docutils.statemachine import StringList, string2lines
+    from docutils._compat import bytes
 except ImportError:
     # The importing module (usually __init__.py in one of the
     # subdirectories) may catch ImportErrors in order to detect the
@@ -108,7 +107,7 @@ class StandardTestCase(unittest.TestCase):
     def setUp(self):
         os.chdir(testroot)
 
-    def failUnlessEqual(self, first, second, msg=None):
+    def assertEqual(self, first, second, msg=None):
         """Fail if the two objects are unequal as determined by the '=='
            operator.
         """
@@ -116,7 +115,7 @@ class StandardTestCase(unittest.TestCase):
             raise self.failureException, \
                   (msg or '%s != %s' % _format_str(first, second))
 
-    def failIfEqual(self, first, second, msg=None):
+    def assertNotEqual(self, first, second, msg=None):
         """Fail if the two objects are equal as determined by the '=='
            operator.
         """
@@ -124,11 +123,15 @@ class StandardTestCase(unittest.TestCase):
             raise self.failureException, \
                   (msg or '%s == %s' % _format_str(first, second))
 
-    # Synonyms for assertion methods
+    # python 2.3
+    if not hasattr(unittest.TestCase, "assertTrue"):
+        assertTrue = unittest.TestCase.failUnless
 
-    assertEqual = assertEquals = failUnlessEqual
+    # aliases for assertion methods, deprecated since Python 2.7
 
-    assertNotEqual = assertNotEquals = failIfEqual
+    failUnlessEqual = assertEquals = assertEqual 
+
+    assertNotEquals = failIfEqual = assertNotEqual
 
 
 class CustomTestCase(StandardTestCase):
@@ -136,16 +139,16 @@ class CustomTestCase(StandardTestCase):
     """
     Helper class, providing extended functionality over unittest.TestCase.
 
-    The methods failUnlessEqual and failIfEqual have been overwritten
+    The methods assertEqual and assertNotEqual have been overwritten
     to provide better support for multi-line strings.  Furthermore,
     see the compare_output method and the parameter list of __init__.
     """
 
-    compare = docutils_difflib.Differ().compare
+    compare = difflib.Differ().compare
     """Comparison method shared by all subclasses."""
 
-    def __init__(self, method_name, input, expected, id, run_in_debugger=0,
-                 suite_settings=None):
+    def __init__(self, method_name, input, expected, id,
+                 run_in_debugger=True, suite_settings=None):
         """
         Initialise the CustomTestCase.
 
@@ -190,14 +193,27 @@ class CustomTestCase(StandardTestCase):
 
     def compare_output(self, input, output, expected):
         """`input`, `output`, and `expected` should all be strings."""
-        if isinstance(input, UnicodeType):
+        if isinstance(input, unicode):
             input = input.encode('raw_unicode_escape')
-        if isinstance(output, UnicodeType):
-            output = output.encode('raw_unicode_escape')
-        if isinstance(expected, UnicodeType):
-            expected = expected.encode('raw_unicode_escape')
+        if sys.version_info > (3,):
+            # API difference: Python 3's node.__str__ doesn't escape
+            #assert expected is None or isinstance(expected, unicode)
+            if isinstance(expected, bytes):
+                expected = expected.decode('utf-8')
+            if isinstance(output, bytes):
+                output = output.decode('utf-8')
+        else:
+            if isinstance(expected, unicode):
+                expected = expected.encode('raw_unicode_escape')
+            if isinstance(output, unicode):
+                output = output.encode('raw_unicode_escape')
+        # Normalize line endings:
+        if expected:
+            expected = '\n'.join(expected.splitlines())
+        if output:
+            output = '\n'.join(output.splitlines())
         try:
-            self.assertEquals(output, expected)
+            self.assertEqual(output, expected)
         except AssertionError, error:
             print >>sys.stderr, '\n%s\ninput:' % (self,)
             print >>sys.stderr, input
@@ -262,7 +278,7 @@ class CustomTestSuite(unittest.TestSuite):
             self.id = id
 
     def addTestCase(self, test_case_class, method_name, input, expected,
-                    id=None, run_in_debugger=0, **kwargs):
+                    id=None, run_in_debugger=False, **kwargs):
         """
         Create a CustomTestCase in the CustomTestSuite.
         Also return it, just in case.
@@ -393,7 +409,7 @@ class TransformTestSuite(CustomTestSuite):
         for name, (transforms, cases) in dict.items():
             for casenum in range(len(cases)):
                 case = cases[casenum]
-                run_in_debugger = 0
+                run_in_debugger = False
                 if len(case)==3:
                     # TODO: (maybe) change the 3rd argument to a dict, so it
                     # can handle more cases by keyword ('disable', 'debug',
@@ -401,7 +417,7 @@ class TransformTestSuite(CustomTestSuite):
                     # But there's also the method that
                     # HtmlPublishPartsTestSuite uses <DJG>
                     if case[2]:
-                        run_in_debugger = 1
+                        run_in_debugger = True
                     else:
                         continue
                 self.addTestCase(
@@ -467,10 +483,10 @@ class ParserTestSuite(CustomTestSuite):
         for name, cases in dict.items():
             for casenum in range(len(cases)):
                 case = cases[casenum]
-                run_in_debugger = 0
+                run_in_debugger = False
                 if len(case)==3:
                     if case[2]:
-                        run_in_debugger = 1
+                        run_in_debugger = True
                     else:
                         continue
                 self.addTestCase(
@@ -484,7 +500,7 @@ class PEPParserTestCase(ParserTestCase):
 
     """PEP-specific parser test case."""
 
-    parser = rst.Parser(rfc2822=1, inliner=rst.states.Inliner())
+    parser = rst.Parser(rfc2822=True, inliner=rst.states.Inliner())
     """Parser shared by all PEPParserTestCases."""
 
     option_parser = frontend.OptionParser(components=(rst.Parser, pep.Reader))
@@ -552,10 +568,10 @@ class GridTableParserTestSuite(CustomTestSuite):
         for name, cases in dict.items():
             for casenum in range(len(cases)):
                 case = cases[casenum]
-                run_in_debugger = 0
+                run_in_debugger = False
                 if len(case) == 4:
                     if case[-1]:
-                        run_in_debugger = 1
+                        run_in_debugger = True
                     else:
                         continue
                 self.addTestCase(self.test_case_class, 'test_parse_table',
@@ -595,10 +611,10 @@ class SimpleTableParserTestSuite(CustomTestSuite):
         for name, cases in dict.items():
             for casenum in range(len(cases)):
                 case = cases[casenum]
-                run_in_debugger = 0
+                run_in_debugger = False
                 if len(case) == 3:
                     if case[-1]:
-                        run_in_debugger = 1
+                        run_in_debugger = True
                     else:
                         continue
                 self.addTestCase(self.test_case_class, 'test_parse',
@@ -612,6 +628,12 @@ class PythonModuleParserTestCase(CustomTestCase):
     def test_parser(self):
         if self.run_in_debugger:
             pdb.set_trace()
+        try:
+            import compiler
+        except ImportError:
+            # skip on Python 3
+            return
+        from docutils.readers.python import moduleparser
         module = moduleparser.parse_module(self.input, 'test data').pformat()
         output = str(module)
         self.compare_output(self.input, output, self.expected)
@@ -619,6 +641,12 @@ class PythonModuleParserTestCase(CustomTestCase):
     def test_token_parser_rhs(self):
         if self.run_in_debugger:
             pdb.set_trace()
+        try:
+            import compiler
+        except ImportError:
+            # skip on Python 3
+            return
+        from docutils.readers.python import moduleparser
         tr = moduleparser.TokenParser(self.input)
         output = tr.rhs(1)
         self.compare_output(self.input, output, self.expected)
@@ -644,10 +672,10 @@ class PythonModuleParserTestSuite(CustomTestSuite):
         for name, cases in dict.items():
             for casenum in range(len(cases)):
                 case = cases[casenum]
-                run_in_debugger = 0
+                run_in_debugger = False
                 if len(case)==3:
                     if case[2]:
-                        run_in_debugger = 1
+                        run_in_debugger = True
                     else:
                         continue
                 self.addTestCase(
@@ -668,7 +696,7 @@ class WriterPublishTestCase(CustomTestCase, docutils.SettingsSpec):
     writer_name = '' # set in subclasses or constructor
 
     def __init__(self, *args, **kwargs):
-        if kwargs.has_key('writer_name'):
+        if 'writer_name' in kwargs:
             self.writer_name = kwargs['writer_name']
             del kwargs['writer_name']
         CustomTestCase.__init__(self, *args, **kwargs)
@@ -700,10 +728,10 @@ class PublishTestSuite(CustomTestSuite):
         for name, cases in dict.items():
             for casenum in range(len(cases)):
                 case = cases[casenum]
-                run_in_debugger = 0
+                run_in_debugger = False
                 if len(case)==3:
                     if case[2]:
-                        run_in_debugger = 1
+                        run_in_debugger = True
                     else:
                         continue
                 self.addTestCase(
@@ -723,10 +751,10 @@ class HtmlPublishPartsTestSuite(CustomTestSuite):
             settings.update(settings_overrides)
             for casenum in range(len(cases)):
                 case = cases[casenum]
-                run_in_debugger = 0
+                run_in_debugger = False
                 if len(case)==3:
                     if case[2]:
-                        run_in_debugger = 1
+                        run_in_debugger = True
                     else:
                         continue
                 self.addTestCase(
@@ -807,19 +835,19 @@ class HtmlWriterPublishPartsTestCase(WriterPublishTestCase):
         output = []
         for key in keys:
             output.append("%r: '''%s'''"
-                          % (key, parts[key].encode('raw_unicode_escape')))
+                          % (key, parts[key]))
             if output[-1].endswith("\n'''"):
                 output[-1] = output[-1][:-4] + "\\n'''"
         return '{' + ',\n '.join(output) + '}\n'
 
 
-def exception_data(code):
+def exception_data(func, *args, **kwds):
     """
-    Execute `code` and return the resulting exception, the exception arguments,
-    and the formatted exception string.
+    Execute `func(*args, **kwds)` and return the resulting exception, the
+    exception arguments, and the formatted exception string.
     """
     try:
-        exec(code)
+        func(*args, **kwds)
     except Exception, detail:
         return (detail, detail.args,
                 '%s: %s' % (detail.__class__.__name__, detail))
@@ -847,16 +875,18 @@ def _format_str(*args):
     return_tuple = []
     for i in args:
         r = repr(i)
-        if ( (isinstance(i, StringType) or isinstance(i, UnicodeType))
+        if ( (isinstance(i, str) or isinstance(i, unicode))
              and '\n' in i):
             stripped = ''
-            if isinstance(i, UnicodeType):
-                # stripped = 'u' or 'U'
+            if isinstance(i, unicode) and r.startswith('u'):
+                stripped = r[0]
+                r = r[1:]
+            elif isinstance(i, bytes) and r.startswith('b'):
                 stripped = r[0]
                 r = r[1:]
             # quote_char = "'" or '"'
             quote_char = r[0]
-            assert quote_char in ("'", '"')
+            assert quote_char in ("'", '"'), quote_char
             assert r[0] == r[-1]
             r = r[1:-1]
             r = (stripped + 3 * quote_char + '\\\n' +
